CKEDITOR.plugins.add("datetime", {
  icons: "datetime",
  init: function (editor) {
    editor.addCommand("datetime", new CKEDITOR.dialogCommand("datetimePicker"))

    editor.ui.addButton("Datetime", {
      label: "Insert \<time\> element",
      command: "datetime",
      toolbar: "insert"
    })

    CKEDITOR.dialog.add("datetimePicker", function (editor) {
      return {
        title: "Select date",
        minWidth: 400,
        minHeight: 200,
        contents: [
          {
            id: "general",
            elements: [
              {
                type: "text",
                id: "selectedTime",
                label: "Time:",
                onShow: function () {
                  // Convert the input to a time input, because CKEditor
                  // doesn't seem to support it.
                  var dialogId = "#" + this.domId
                  var inputId = "#" + jQuery(dialogId).find("input").attr("id");

                  var $timeInput = jQuery(inputId);

                  $timeInput.attr("type", "time");

                  // Changing the type means the input loses styling, so re-add it.
                  $timeInput.css({
                    "border": "1px solid #b8b8b8",
                    "border-radius": "2px"
                  });

                  // CKEditor wraps things in tables - make this row half-width.
                  $timeInput.closest("tr").css({
                    "width": "50%",
                    "display": "inline-block",
                  });
                }
              },
              {
                // Add a text input to store the date.
                type: "text",
                id: "selectedDate",
                label: "Date:",
                onShow: function () {

                  var dialogId = "#" + this.domId
                  var inputId = "#" + jQuery(dialogId).find("input").attr("id")

                  // CKEditor wraps things in tables - make this row half-width.
                  jQuery(inputId).closest("tr").css({
                    "width": "50%",
                    "display": "inline-block",
                    "vertical-align": "top"
                  });

                  // Apply the datepicker to the input control
                  jQuery(dialogId).datepicker({
                    changeMonth: true,
                    changeYear: true,
                    // TODO: integrate with Drupal date formats.
                    dateFormat: "yy-mm-dd",
                    showButtonPanel: true,
                    altField: inputId
                  });
                }
              }
            ]
          }
        ],
        onOk: function () {
          // TODO: integrate with Drupal date formats.
          var dateOptions = {
            "day": "numeric",
            "month": "short",
            "year": "numeric"
          };

          // Default to midnight.
          var selectedTime = "00:00";

          // If a time is selected, show the time.
          var timeValue = this.getValueOf("general", "selectedTime");
          if (!!timeValue) {
            selectedTime = timeValue;
            dateOptions.hour = "numeric";
            dateOptions.minute = "numeric";
          }

          // Get the selected date from the datepicker.
          var dateValue = this.getValueOf("general", "selectedDate");

          // TODO: deal with time zones.
          var selectedDate = new Date(dateValue + "T" + selectedTime);
          var machineDate = selectedDate.toISOString();

          // TODO: get locale from Drupal.
          var locale = "en-GB";

          // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toLocaleDateString
          var humanDate = selectedDate.toLocaleString(locale, dateOptions);



          editor.insertHtml('<time datetime="' + machineDate + '">' + humanDate + '</time>')
        }
      }
    })
  }
})
